from chisubmit.tests.fixtures import users_and_courses, complete_course
import unittest

#class UsersAndCoursesFixtureCheck(ChisubmitFixtureTestCase, unittest.TestCase):
#        
#    FIXTURE = users_and_courses 
#    
#    
#class CompleteCourseFixtureCheck(ChisubmitFixtureTestCase, unittest.TestCase):
#        
#    FIXTURE = complete_course     