from chisubmit.backend.webapp.api.courses import views
from chisubmit.backend.webapp.api.users import views
from chisubmit.backend.webapp.api.assignments import views
from chisubmit.backend.webapp.api.teams import views
